# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Shared resource args for Cloud API Gateway surface."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


def LocationAttributeConfig(default=None):
  """Creates location attribute config."""
  default_keyword = default
  if default == '-':
    default_keyword = 'a wildcard'

  fallthroughs = []
  if default:
    fallthroughs.append(
        deps.Fallthrough(
            lambda: default,
            'Location for API and API Configs. Defaults to {}'.format(
                default_keyword)))

  return concepts.ResourceParameterAttributeConfig(
      name='location',
      fallthroughs=fallthroughs,
      help_text='Cloud location for {resource}.')


def GatewayAttributeConfig(name='gateway'):
  return concepts.ResourceParameterAttributeConfig(
      name=name,
      help_text='Name for API Gateway')


def ApiAttributeConfig(name='api', wildcard=False):
  fallthroughs = []
  if wildcard:
    fallthroughs.append(
        deps.Fallthrough(
            lambda: '-',
            'Defaults to wildcard for all APIs'))

  return concepts.ResourceParameterAttributeConfig(
      name=name,
      fallthroughs=fallthroughs,
      help_text='API ID.')


def ApiConfigAttributeConfig(name='api-config'):
  return concepts.ResourceParameterAttributeConfig(
      name=name,
      help_text='API Config ID.')


def OperationAttributeConfig(name='operation'):
  return concepts.ResourceParameterAttributeConfig(
      name=name,
      help_text='The name for the API Gateway operation')


def GetLocationResourceSpec(resource_name='location', default=None):
  return concepts.ResourceSpec(
      'apigateway.projects.locations',
      resource_name=resource_name,
      locationsId=LocationAttributeConfig(default=default),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG)


def GetGatewayResourceSpec(resource_name='gateway'):
  return concepts.ResourceSpec(
      'apigateway.projects.locations.gateways',
      resource_name=resource_name,
      gatewaysId=GatewayAttributeConfig(),
      locationsId=LocationAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG)


def GetApiResourceSpec(resource_name='api', wildcard=False):
  return concepts.ResourceSpec(
      'apigateway.projects.locations.apis',
      resource_name=resource_name,
      apisId=ApiAttributeConfig(wildcard=wildcard),
      locationsId=LocationAttributeConfig(default='global'),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG)


def GetApiConfigResourceSpec(resource_name='api-config'):
  return concepts.ResourceSpec(
      'apigateway.projects.locations.apis.configs',
      resource_name=resource_name,
      configsId=ApiConfigAttributeConfig(),
      apisId=ApiAttributeConfig(),
      locationsId=LocationAttributeConfig(default='global'),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG)


def GetOperationResourceSpec(resource_name='operation'):
  return concepts.ResourceSpec(
      'apigateway.projects.locations.operations',
      resource_name=resource_name,
      operationsId=OperationAttributeConfig(),
      locationsId=LocationAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG)


def AddGatewayResourceArg(parser, verb, positional=False, required=True):
  """Adds Gateway resource argument to parser.

  Args:
    parser: parser to add arg to
    verb: action being taken with the Gateway
    positional: Boolean indicating if argument is positional, default False
    required: Boolean for if this is required, default is True

  Returns: None
  """

  if positional:
    name = 'gateway'
  else:
    name = '--gateway'
  concept_parsers.ConceptParser.ForResource(
      name,
      GetGatewayResourceSpec(),
      'Name for gateway which will be {}.'.format(verb),
      required=required).AddToParser(parser)


def AddGatewayApiConfigResourceArgs(parser, verb, gateway_required=True,
                                    api_config_required=True):
  """Adds Gateway and API Config resource arguments to parser.

  Args:
    parser: parser to add arg to
    verb: action being taken with the Gateway
    gateway_required: Boolean for if Gateway is required, default is True
    api_config_required: Boolean for if API Config is required, default is True

  Returns: None
  """

  concept_parsers.ConceptParser(
      [
          presentation_specs.ResourcePresentationSpec(
              'gateway',
              GetGatewayResourceSpec(),
              'Name for gateway which will be {}.'.format(verb),
              required=gateway_required),
          presentation_specs.ResourcePresentationSpec(
              '--api-config',
              GetApiConfigResourceSpec(),
              'Resource name for API config the gateway will use.',
              flag_name_overrides={'location': ''},
              required=api_config_required)
      ]).AddToParser(parser)


def AddLocationResourceArg(parser, verb, positional=False, default=None,
                           required=True):
  """Adds location resource argument to parser.

  Args:
    parser: parser to add arg to
    verb: action being taken with the location
    positional: Optional boolean indiicating if argument is positional
    default: Optional default value for the arg
    required: Boolean for if this is required, default is True

  Returns: None
  """

  if positional:
    name = 'location'
  else:
    name = '--location'

  override = None
  if default == 'global':
    override = {'location': ''}

  concept_parsers.ConceptParser.ForResource(
      name,
      GetLocationResourceSpec(default=default),
      'Parent location which {}.'.format(verb),
      flag_name_overrides=override,
      required=required).AddToParser(parser)


def AddApiResourceArg(parser, verb, positional=False, required=True,
                      wildcard=False):
  """Adds API resource argument to parser.

  Args:
    parser: parser to add arg to
    verb: action being taken with the API
    positional: Optional boolean indiicating if argument is positional
    required: Boolean for if this is required, default is True
    wildcard: Boolean. Does arg have a default wildcard? default: False

  Returns: None
  """

  if positional:
    name = 'api'
  else:
    name = '--api'
  concept_parsers.ConceptParser.ForResource(
      name,
      GetApiResourceSpec(wildcard=wildcard),
      'Name for API which {}.'.format(verb),
      flag_name_overrides={'location': ''},
      required=required).AddToParser(parser)


def AddApiConfigResourceArg(parser, verb, positional=False, required=True):
  """Adds API Config resource argument to parser.

  Args:
    parser: parser to add arg to
    verb: action being taken with the API Config
    positional: Boolean indicating if argument is positional, default False
    required: Boolean for if this is required, default is True

  Returns: None
  """

  if positional:
    name = 'api_config'
  else:
    name = '--api-config'
  concept_parsers.ConceptParser.ForResource(
      name,
      GetApiConfigResourceSpec(),
      'Name for API Config which will be {}.'.format(verb),
      flag_name_overrides={'location': ''},
      required=required).AddToParser(parser)


def AddOperationResourceArgs(parser, verb):
  concept_parsers.ConceptParser.ForResource(
      'operation',
      GetOperationResourceSpec(),
      'The name of the operation to {}.'.format(verb),
      required=True).AddToParser(parser)



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Runtime-config resource transforms and symbols dict.

NOTICE: Each TransformFoo() method is the implementation of a foo() transform
function. Even though the implementation here is in Python the usage in resource
projection and filter expressions is language agnostic. This affects the
Pythonicness of the Transform*() methods:
  (1) The docstrings are used to generate external user documentation.
  (2) The method prototypes are included in the documentation. In particular the
      prototype formal parameter names are stylized for the documentation.
  (3) The types of some args, like r, are not fixed until runtime. Other args
      may have either a base type value or string representation of that type.
      It is up to the transform implementation to silently do the string=>type
      conversions. That's why you may see e.g. int(arg) in some of the methods.
  (4) Unless it is documented to do so, a transform function must not raise any
      exceptions. The `undefined' arg is used to handle all unusual conditions,
      including ones that would raise exceptions.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

# The DEADLINE_EXCEEDED error code.
DEADLINE_EXCEEDED = 4


def TransformWaiterStatus(r, undefined=''):
  """Returns a short description of the status of a waiter or waiter operation.

  Status will be one of WAITING, SUCCESS, FAILURE, or TIMEOUT.

  Args:
    r: a JSON-serializable object
    undefined: Returns this value if the resource status cannot be determined.

  Returns:
    One of WAITING, SUCCESS, FAILURE, or TIMEOUT

  Example:
    `--format="table(name, status())"`:::
    Displays the status in table column two.
  """
  if not isinstance(r, dict):
    return undefined

  if not r.get('done'):
    return 'WAITING'

  error = r.get('error')
  if not error:
    return 'SUCCESS'

  if error.get('code') == DEADLINE_EXCEEDED:
    return 'TIMEOUT'
  else:
    return 'FAILURE'


_TRANSFORMS = {
    'waiter_status': TransformWaiterStatus,
}


def GetTransforms():
  """Returns the runtimeconfig-specific resource transform symbol table."""
  return _TRANSFORMS



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""'logging settings describe' command."""


from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.logging import util
from googlecloudsdk.calliope import base


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Describe(base.DescribeCommand):
  # pylint: disable=line-too-long
  """Displays the settings for the Cloud Logging Logs Router.

  If *kmsKeyName* is present in the output, then CMEK is enabled for your
  project, folder, organization or billing-account. You can also find the Logs
  Router service account using this command.

  ## EXAMPLES

  To describe the Logs Router settings for a project, run:

    $ {command} --project=[PROJECT_ID]

  To describe the Logs Router settings for an organization, run:

    $ {command} --organization=[ORGANIZATION_ID]

    kmsKeyName:
    'projects/my-project/locations/my-location/keyRings/my-keyring/cryptoKeys/key'
    name: 'organizations/[ORGANIZATION_ID]/settings'
    serviceAccountId:
    '[SERVICE_ACCOUNT_ID]@gcp-sa-logging.iam.gserviceaccount.com'
  """

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    util.AddParentArgs(parser, 'Describe settings')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      The settings for the specified project, folder, organizations or
      billing-account.
    """
    parent_name = util.GetParentFromArgs(args)
    return util.GetClient().projects.GetSettings(
        util.GetMessages().LoggingProjectsGetSettingsRequest(name=parent_name))



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for gkeonprem API clients for VMware cluster resources."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis


class ClustersClient(object):
  """Client for clusters in gkeonprem vmware API."""

  def __init__(self, client=None, messages=None):
    self.client = client or apis.GetClientInstance('gkeonprem', 'v1')
    self.messages = messages or self.client.MESSAGES_MODULE
    self._service = self.client.projects_locations_vmwareClusters

  def List(self, location_ref, limit=None, page_size=100):
    """Lists Clusters in the GKE On-Prem VMware API."""
    list_req = self.messages.GkeonpremProjectsLocationsVmwareClustersListRequest(
        parent=location_ref.RelativeName())
    return list_pager.YieldFromList(
        self._service,
        list_req,
        field='vmwareClusters',
        batch_size=page_size,
        limit=limit,
        batch_size_attribute='pageSize')

  def Describe(self, resource_ref):
    """Gets a gkeonprem API cluster resource."""
    req = self.messages.GkeonpremProjectsLocationsVmwareClustersGetRequest(
        name=resource_ref.RelativeName())
    return self._service.Get(req)

  def Enroll(self, admin_cluster_membership_ref, resource_ref, local_name=None):
    """Enrolls a VMware cluster to Anthos."""
    enroll_vmware_cluster_request = self.messages.EnrollVmwareClusterRequest(
        adminClusterMembership=admin_cluster_membership_ref.RelativeName(),
        vmwareClusterId=resource_ref.Name(),
        localName=local_name,
    )
    req = self.messages.GkeonpremProjectsLocationsVmwareClustersEnrollRequest(
        parent=resource_ref.Parent().RelativeName(),
        enrollVmwareClusterRequest=enroll_vmware_cluster_request,
    )
    return self._service.Enroll(req)

  def Unenroll(self, resource_ref, force=False):
    """Unenrolls an Anthos cluster on VMware."""
    req = self.messages.GkeonpremProjectsLocationsVmwareClustersUnenrollRequest(
        name=resource_ref.RelativeName(),
        force=force,
    )
    return self._service.Unenroll(req)

  def Delete(self,
             resource_ref,
             allow_missing=False,
             validate_only=False,
             force=False):
    """Deletes an Anthos cluster on VMware."""
    req = self.messages.GkeonpremProjectsLocationsVmwareClustersDeleteRequest(
        name=resource_ref.RelativeName(),
        allowMissing=allow_missing,
        validateOnly=validate_only,
        force=force,
    )
    return self._service.Delete(req)

  def Create(self, resource_ref, admin_cluster_membership_ref, args):
    """Creates an Anthos cluster on VMware."""
    load_balancer = self.messages.VmwareLoadBalancerConfig(
        vipConfig=self.messages.VmwareVipConfig(
            controlPlaneVip=args.control_plane_vip,
            ingressVip=args.ingress_vip,
        ),)

    if args.metal_lb_config_address_pools:
      address_pools = []
      for address_pool in args.metal_lb_config_address_pools:
        address_pools.append(
            self.messages.VmwareAddressPool(
                pool=address_pool['pool'],
                addresses=address_pool['addresses'],
                avoidBuggyIps=address_pool.get('avoid_buggy_ips', False),
                manualAssign=address_pool.get('manual_assign', False),
            ),
        )
      load_balancer.metalLbConfig = self.messages.VmwareMetalLbConfig(
          addressPools=address_pools)

    if args.f5_config_address:
      load_balancer.f5Config = self.messages.VmwareF5BigIpConfig(
          address=args.f5_config_address,
          partition=args.f5_config_partition,
          snatPool=args.f5_config_snat_pool,
      )

    vmware_cluster = self.messages.VmwareCluster(
        name=resource_ref.RelativeName(),
        adminClusterMembership=admin_cluster_membership_ref.RelativeName(),
        onPremVersion=args.version,
        networkConfig=self.messages.VmwareNetworkConfig(
            serviceAddressCidrBlocks=args.service_address_cidr_blocks,
            podAddressCidrBlocks=args.pod_address_cidr_blocks,
        ),
        loadBalancer=load_balancer,
    )

    req = self.messages.GkeonpremProjectsLocationsVmwareClustersCreateRequest(
        parent=resource_ref.Parent().RelativeName(),
        validateOnly=args.validate_only,
        vmwareCluster=vmware_cluster,
        vmwareClusterId=resource_ref.Name(),
    )
    return self._service.Create(req)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Provides helper methods for dealing with JSON files for Spanner IAM."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.spanner import databases
from googlecloudsdk.api_lib.spanner import instances
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.command_lib.iam import iam_util


def AddInstanceIamPolicyBinding(instance_ref, member, role):
  """Adds a policy binding to an instance IAM policy."""
  msgs = apis.GetMessagesModule('spanner', 'v1')
  policy = instances.GetIamPolicy(instance_ref)
  iam_util.AddBindingToIamPolicy(msgs.Binding, policy, member, role)
  return instances.SetPolicy(instance_ref, policy)


def SetInstanceIamPolicy(instance_ref, policy):
  """Sets the IAM policy on an instance."""
  msgs = apis.GetMessagesModule('spanner', 'v1')
  policy, field_mask = iam_util.ParsePolicyFileWithUpdateMask(policy,
                                                              msgs.Policy)
  return instances.SetPolicy(instance_ref, policy, field_mask)


def RemoveInstanceIamPolicyBinding(instance_ref, member, role):
  """Removes a policy binding from an instance IAM policy."""
  policy = instances.GetIamPolicy(instance_ref)
  iam_util.RemoveBindingFromIamPolicy(policy, member, role)
  return instances.SetPolicy(instance_ref, policy)


def AddDatabaseIamPolicyBinding(database_ref, member, role):
  """Adds a policy binding to a database IAM policy."""
  msgs = apis.GetMessagesModule('spanner', 'v1')
  policy = databases.GetIamPolicy(database_ref)
  iam_util.AddBindingToIamPolicy(msgs.Binding, policy, member, role)
  return databases.SetPolicy(database_ref, policy)


def SetDatabaseIamPolicy(database_ref, policy):
  """Sets the IAM policy on a database."""
  msgs = apis.GetMessagesModule('spanner', 'v1')
  policy = iam_util.ParsePolicyFile(policy, msgs.Policy)
  return databases.SetPolicy(database_ref, policy)


def RemoveDatabaseIamPolicyBinding(database_ref, member, role):
  """Removes a policy binding from a database IAM policy."""
  policy = databases.GetIamPolicy(database_ref)
  iam_util.RemoveBindingFromIamPolicy(policy, member, role)
  return databases.SetPolicy(database_ref, policy)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for flags for `gcloud tasks` commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.compute.networks import flags as compute_network_flags
from googlecloudsdk.command_lib.compute.networks.subnets import flags as compute_subnet_flags
from googlecloudsdk.command_lib.kms import resource_args as kms_resource_args
from googlecloudsdk.command_lib.notebooks import completers
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import properties


def GetEnvironmentResourceArg(help_text, positional=True, required=True):
  """Constructs and returns the Environment Resource Argument."""

  def GetEnvironmentResourceSpec():
    """Constructs and returns the Resource specification for Environment."""

    def EnvironmentAttributeConfig():
      return concepts.ResourceParameterAttributeConfig(
          name='environment', help_text=help_text)

    def LocationAttributeConfig():
      return concepts.ResourceParameterAttributeConfig(
          name='{}location'.format('' if positional else 'environment-'),
          help_text=(
              'Google Cloud location of this environment '
              'https://cloud.google.com/compute/docs/regions-zones/#locations.'
          ),
          completer=completers.LocationCompleter,
          fallthroughs=[
              deps.ArgFallthrough('--location'),
              deps.PropertyFallthrough(properties.VALUES.notebooks.location)
          ],
      )

    return concepts.ResourceSpec(
        'notebooks.projects.locations.environments',
        resource_name='environment',
        environmentsId=EnvironmentAttributeConfig(),
        locationsId=LocationAttributeConfig(),
        projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
        disable_auto_completers=False)

  return concept_parsers.ConceptParser.ForResource(
      '{}environment'.format('' if positional else '--'),
      GetEnvironmentResourceSpec(),
      help_text,
      required=required)


def AddListEnvironmentFlags(parser):
  parser.add_argument(
      '--location',
      completer=completers.LocationCompleter,
      help=('Google Cloud location of this environment: '
            'https://cloud.google.com/compute/docs/regions-zones/#locations.'))


def AddCreateEnvironmentFlags(parser):
  """Construct groups and arguments specific to the environment create."""
  source_group = parser.add_group(mutex=True, required=True)
  vm_source_group = source_group.add_group()
  vm_mutex_group = vm_source_group.add_group(mutex=True, required=True)
  container_group = source_group.add_group()
  GetEnvironmentResourceArg(
      ('User-defined unique name of this environment. The environment name '
       'must be 1 to 63 characters long and contain only lowercase letters, '
       'numeric characters, and dashes. The first character must be a lowercase'
       'letter and the last character cannot be a dash.')).AddToParser(parser)
  parser.add_argument(
      '--description', help='A brief description of this environment.')
  parser.add_argument('--display-name', help='Name to display on the UI.')
  parser.add_argument(
      '--post-startup-script',
      help=(
          'Path to a Bash script that automatically runs after a notebook '
          'instance fully boots up. The path must be a URL or Cloud Storage path'
          '(gs://`path-to-file/`file-name`).'))
  base.ASYNC_FLAG.AddToParser(parser)
  vm_source_group.add_argument(
      '--vm-image-project',
      help=('The ID of the Google Cloud project that this VM image belongs to.'
            'Format: projects/`{project_id}`.'),
      default='deeplearning-platform-release')
  vm_mutex_group.add_argument(
      '--vm-image-family',
      help=('Use this VM image family to find the image; the newest image in '
            'this family will be used.'),
      default='common-cpu')
  vm_mutex_group.add_argument(
      '--vm-image-name', help='Use this VM image name to find the image.')
  container_group.add_argument(
      '--container-repository',
      help=('The path to the container image repository. For example: '
            'gcr.io/`{project_id}`/`{image_name}`.'),
      required=True)
  container_group.add_argument(
      '--container-tag',
      help='The tag of the container image. If not specified, this defaults to the latest tag.'
  )


def AddDeleteEnvironmentFlags(parser):

  GetEnvironmentResourceArg(
      ('User-defined unique name of this environment. The environment name '
       'must be 1 to 63 characters long and contain only lowercase letters, '
       'numeric characters, and dashes. The first character must be a lowercase'
       'letter and the last character cannot be a dash.')).AddToParser(parser)
  base.ASYNC_FLAG.AddToParser(parser)


def AddDescribeEnvironmentFlags(parser):
  GetEnvironmentResourceArg(
      ('User-defined unique name of this environment. The environment name '
       'must be 1 to 63 characters long and contain only lowercase letters, '
       'numeric characters, and dashes. The first character must be a lowercase'
       'letter and the last character cannot be a dash.')).AddToParser(parser)


def GetLocationResourceArg(help_text):
  """Constructs and returns the Location Resource Argument."""

  def GetLocationResourceSpec():
    """Constructs and returns the Location Resource Argument."""
    def LocationAttributeConfig():
      return concepts.ResourceParameterAttributeConfig(
          name='location',
          help_text=(
              'Google Cloud location of this runtime '
              'https://cloud.google.com/compute/docs/regions-zones/#locations.'
          ),
          fallthroughs=[
              deps.PropertyFallthrough(properties.VALUES.notebooks.location),
          ],
      )

    return concepts.ResourceSpec(
        'notebooks.projects.locations',
        resource_name='location',
        projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
        locationsId=LocationAttributeConfig())

  return concept_parsers.ConceptParser.ForResource(
      '--location', GetLocationResourceSpec(), help_text, required=True)


def GetRuntimeResourceArg(help_text):
  """Constructs and returns the Runtime Resource Argument."""

  def GetRuntimeResourceSpec():
    """Constructs and returns the Resource specification for Runtime."""

    def RuntimeAttributeConfig():
      return concepts.ResourceParameterAttributeConfig(
          name='rumtime', help_text=help_text)

    def LocationAttributeConfig():
      return concepts.ResourceParameterAttributeConfig(
          name='location',
          help_text=(
              'Google Cloud location of this runtime '
              'https://cloud.google.com/compute/docs/regions-zones/#locations.'
          ),
          fallthroughs=[
              deps.PropertyFallthrough(properties.VALUES.notebooks.location),
          ],
      )

    return concepts.ResourceSpec(
        'notebooks.projects.locations.runtimes',
        resource_name='runtime',
        projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
        locationsId=LocationAttributeConfig(),
        runtimesId=RuntimeAttributeConfig(),
        disable_auto_completers=False)

  return concept_parsers.ConceptParser.ForResource(
      'runtime', GetRuntimeResourceSpec(), help_text, required=True)


def GetInstanceResourceArg(help_text):
  """Constructs and returns the Instance Resource Argument."""

  def GetInstanceResourceSpec():
    """Constructs and returns the Resource specification for Instance."""

    def InstanceAttributeConfig():
      return concepts.ResourceParameterAttributeConfig(
          name='instance', help_text=help_text)

    def LocationAttributeConfig():
      return concepts.ResourceParameterAttributeConfig(
          name='location',
          help_text=(
              'Google Cloud location of this environment '
              'https://cloud.google.com/compute/docs/regions-zones/#locations.'
          ),
          fallthroughs=[
              deps.PropertyFallthrough(properties.VALUES.notebooks.location),
          ],
      )

    return concepts.ResourceSpec(
        'notebooks.projects.locations.instances',
        resource_name='instance',
        instancesId=InstanceAttributeConfig(),
        locationsId=LocationAttributeConfig(),
        projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
        disable_auto_completers=False)

  return concept_parsers.ConceptParser.ForResource(
      'instance', GetInstanceResourceSpec(), help_text, required=True)


def AddNetworkArgument(help_text, parser):
  """Adds Resource arg for network to the parser."""

  def GetNetworkResourceSpec():
    """Constructs and returns the Resource specification for Subnet."""

    def NetworkAttributeConfig():
      return concepts.ResourceParameterAttributeConfig(
          name='network',
          help_text=help_text,
          completer=compute_network_flags.NetworksCompleter)

    return concepts.ResourceSpec(
        'compute.networks',
        resource_name='network',
        network=NetworkAttributeConfig(),
        project=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
        disable_auto_completers=False)

  concept_parsers.ConceptParser.ForResource('--network',
                                            GetNetworkResourceSpec(),
                                            help_text).AddToParser(parser)


def AddSubnetArgument(help_text, parser):
  """Adds Resource arg for subnetwork to the parser."""

  def GetSubnetResourceSpec():
    """Constructs and returns the Resource specification for Subnet."""

    def SubnetAttributeConfig():
      return concepts.ResourceParameterAttributeConfig(
          name='subnet',
          help_text=help_text,
          completer=compute_subnet_flags.SubnetworksCompleter)

    def RegionAttributeConfig():
      return concepts.ResourceParameterAttributeConfig(
          name='subnet-region',
          help_text=(
              'Google Cloud region of this subnetwork '
              'https://cloud.google.com/compute/docs/regions-zones/#locations.'
          ),
          completer=completers.RegionCompleter)

    return concepts.ResourceSpec(
        'compute.subnetworks',
        resource_name='subnetwork',
        subnetwork=SubnetAttributeConfig(),
        region=RegionAttributeConfig(),
        project=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
        disable_auto_completers=False)

  concept_parsers.ConceptParser.ForResource('--subnet', GetSubnetResourceSpec(),
                                            help_text).AddToParser(parser)


def AddRuntimeResource(parser, add_async_flag=True):
  GetRuntimeResourceArg(
      ('User-defined unique name of this runtime. The runtime name must be '
       '1 to 63 characters long and contain only lowercase letters, numeric '
       'characters, and dashes. The first character must be a lowercase letter '
       'and the last character cannot be a dash.')).AddToParser(parser)
  if add_async_flag:
    base.ASYNC_FLAG.AddToParser(parser)


def AddInstanceResource(parser, add_async_flag=True):
  GetInstanceResourceArg(
      ('User-defined unique name of this instance. The instance name must be '
       '1 to 63 characters long and contain only lowercase letters, numeric '
       'characters, and dashes. The first character must be a lowercase letter '
       'and the last character cannot be a dash.')).AddToParser(parser)
  if add_async_flag:
    base.ASYNC_FLAG.AddToParser(parser)


def AddCreateInstanceFlags(parser):
  """Construct groups and arguments specific to the instance creation."""
  accelerator_choices = [
      'NVIDIA_TESLA_A100', 'NVIDIA_TESLA_K80', 'NVIDIA_TESLA_P100',
      'NVIDIA_TESLA_V100', 'NVIDIA_TESLA_P4', 'NVIDIA_TESLA_T4',
      'NVIDIA_TESLA_T4_VWS', 'NVIDIA_TESLA_P100_VWS', 'NVIDIA_TESLA_P4_VWS',
      'TPU_V2', 'TPU_V3'
  ]
  boot_disk_choices = ['PD_STANDARD', 'PD_SSD', 'PD_BALANCED']
  encryption_choices = ['GMEK', 'CMEK']
  AddInstanceResource(parser)
  environment_group = parser.add_group(mutex=True, required=True)
  GetEnvironmentResourceArg((
      'User-defined unique name of this environment. The environment name '
      'must be 1 to 63 characters long and contain only lowercase letters, '
      'numeric characters, and dashes. The first character must be a lowercase '
      'letter and the last character cannot be a dash.'),
                            positional=False,
                            required=False).AddToParser(environment_group)
  vm_source_group = environment_group.add_group()
  vm_mutex_group = vm_source_group.add_group(mutex=True, required=True)
  container_group = environment_group.add_group()
  vm_source_group.add_argument(
      '--vm-image-project',
      help=('The ID of the Google Cloud project that this VM image belongs to. '
            'Format: projects/`{project_id}`.'),
      default='deeplearning-platform-release')
  vm_mutex_group.add_argument(
      '--vm-image-family',
      help=('Use this VM image family to find the image; the newest image in '
            'this family will be used.'),
      default='common-cpu')
  vm_mutex_group.add_argument(
      '--vm-image-name', help='Use this VM image name to find the image.')
  container_group.add_argument(
      '--container-repository',
      help=('The path to the container image repository. '
            'For example: gcr.io/`{project_id}`/`{image_name}`.'),
      required=True)
  container_group.add_argument(
      '--container-tag',
      help=('The tag of the container image. If not specified, '
            'this defaults to the latest tag.'))
  parser.add_argument(
      '--post-startup-script',
      help=(
          'Path to a Bash script that automatically runs after a notebook '
          'instance fully boots up. The path must be a URL or Cloud Storage path '
          '(gs://`path-to-file`/`file-name`).'))
  parser.add_argument(
      '--service-account',
      help=(
          'The service account on this instance, giving access to other '
          'Google Cloud services. You can use any service account within the '
          'same project, but you must have the service account user permission '
          'to use the instance. If not specified, the [Compute Engine default '
          'service account](/compute/docs/access/service-accounts#default_'
          'service_account) is used.'))
  parser.add_argument(
      '--machine-type',
      help=(
          'The '
          '[Compute Engine machine type](https://cloud.google.com/sdk/gcloud/reference/compute/machine-types) '
          'of this instance.'),
      default='n1-standard-1')
  parser.add_argument(
      '--instance-owners',
      help=(
          'The owners of this instance after creation. Format: '
          '`alias@example.com`. Currently supports one owner only. If not specified'
          ", all of the service account users of the VM instance\'s service "
          'account can use the instance.'))
  accelerator_group = parser.add_group(
      help=(
          'The hardware accelerator used on this instance. If you use '
          'accelerators, make sure that your configuration has [enough vCPUs and '
          'memory to support the `machine_type` you have selected](/compute/'
          'docs/gpus/#gpus-list).'))
  accelerator_group.add_argument(
      '--accelerator-type',
      help='Type of this accelerator.',
      choices=accelerator_choices,
      default=None)
  accelerator_group.add_argument(
      '--accelerator-core-count',
      help='Count of cores of this accelerator.',
      type=int)
  gpu_group = parser.add_group(help='GPU driver configurations.')
  gpu_group.add_argument(
      '--install-gpu-driver',
      action='store_true',
      dest='install_gpu_driver',
      help=(
          'Whether the end user authorizes Google Cloud to install a GPU '
          'driver on this instance. If this field is empty or set to false, the '
          'GPU driver won\'t be installed. Only applicable to instances with GPUs.'
      ))
  gpu_group.add_argument(
      '--custom-gpu-driver-path',
      help=(
          'Specify a custom Cloud Storage path where the GPU driver is '
          'stored. If not specified, we\'ll automatically choose from official '
          'GPU drivers.'))
  boot_group = parser.add_group(help='Boot disk configurations.')
  boot_group.add_argument(
      '--boot-disk-type',
      choices=boot_disk_choices,
      default=None,
      help=('The type of disk attached to this instance, defaults to standard '
            'persistent disk (`PD_STANDARD`).'))
  boot_group.add_argument(
      '--boot-disk-size',
      type=int,
      help=('The size of the disk in GB attached to this instance, up to a '
            'maximum of 64000 GB (64 TB). The minimum recommended value '
            'is 100 GB. If not specified, this defaults to 100.'))
  encryption_group = parser.add_group(help='Disk encryption configurations.')
  encryption_group.add_argument(
      '--disk-encryption',
      choices=encryption_choices,
      default=None,
      help='Disk encryption method used on the boot disk, defaults to GMEK.')
  kms_resource_args.AddKmsKeyResourceArg(encryption_group, 'instance')
  network_group = parser.add_group(help='Network configs.')
  network_group.add_argument(
      '--no-public-ip',
      action='store_true',
      dest='no_public_ip',
      help="""\
  If specified, no public IP will be assigned to this instance.""")
  network_group.add_argument(
      '--no-proxy-access',
      action='store_true',
      dest='no_proxy_access',
      help="""\
  If true, the notebook instance will not register with the proxy.""")

  AddNetworkArgument(
      ('The name of the VPC that this instance is in. Format: '
       'projects/`{project_id}`/global/networks/`{network_id}`.'),
      network_group)
  AddSubnetArgument(
      ('The name of the subnet that this instance is in. Format: projects/'
       '`{project_id}`/regions/`{region}`/subnetworks/`{subnetwork_id}`.'),
      network_group)
  parser.add_argument(
      '--labels',
      help=('Labels to apply to this instance. These can be later modified '
            'by the setLabels method.'),
      type=arg_parsers.ArgDict(),
      metavar='KEY=VALUE')
  parser.add_argument(
      '--metadata',
      help='Custom metadata to apply to this instance.',
      type=arg_parsers.ArgDict(),
      metavar='KEY=VALUE')


def AddDescribeInstanceFlags(parser):
  AddInstanceResource(parser, add_async_flag=False)


def AddDeleteInstanceFlags(parser):
  AddInstanceResource(parser)


def AddListInstanceFlags(parser):
  parser.add_argument(
      '--location',
      completer=completers.LocationCompleter,
      help=('Google Cloud location of this environment '
            'https://cloud.google.com/compute/docs/regions-zones/#locations.'))


def AddRegisterInstanceFlags(parser):
  AddInstanceResource(parser)


def AddResetInstanceFlags(parser):
  AddInstanceResource(parser)


def AddStartInstanceFlags(parser):
  AddInstanceResource(parser)


def AddStopInstanceFlags(parser):
  AddInstanceResource(parser)


def AddGetHealthInstanceFlags(parser):
  AddInstanceResource(parser, add_async_flag=False)


def AddIsUpgradeableInstanceFlags(parser):
  AddInstanceResource(parser, add_async_flag=False)


def AddUpgradeInstanceFlags(parser):
  AddInstanceResource(parser)


def AddRollbackInstanceFlags(parser):
  AddInstanceResource(parser)
  parser.add_argument(
      '--target-snapshot',
      help='The saved snapshot to rollback to',
      required=True)


def AddUpdateInstanceFlags(parser):
  """Adds accelerator, labels and machine type flags to the parser for update."""
  accelerator_choices = [
      'NVIDIA_TESLA_A100', 'NVIDIA_TESLA_K80', 'NVIDIA_TESLA_P100',
      'NVIDIA_TESLA_V100', 'NVIDIA_TESLA_P4', 'NVIDIA_TESLA_T4',
      'NVIDIA_TESLA_T4_VWS', 'NVIDIA_TESLA_P100_VWS', 'NVIDIA_TESLA_P4_VWS',
      'TPU_V2', 'TPU_V3'
  ]
  AddInstanceResource(parser)
  update_group = parser.add_group(required=True)
  update_group.add_argument(
      '--accelerator-type',
      help='Type of this accelerator.',
      choices=accelerator_choices,
      default=None)
  update_group.add_argument(
      '--accelerator-core-count',
      help='Count of cores of this accelerator.',
      type=int)
  update_group.add_argument(
      '--labels',
      help=('Labels to apply to this instance. '
            'These can be later modified by the setLabels method.'),
      type=arg_parsers.ArgDict(),
      metavar='KEY=VALUE')
  update_group.add_argument(
      '--machine-type',
      help='The [Compute Engine machine type](/compute/docs/machine-types).')


def AddDeleteRuntimeFlags(parser):
  AddRuntimeResource(parser)


def AddDescribeRuntimeFlags(parser):
  AddRuntimeResource(parser, add_async_flag=False)


def AddStartRuntimeFlags(parser):
  AddRuntimeResource(parser)


def AddStopRuntimeFlags(parser):
  AddRuntimeResource(parser)


def AddSwitchRuntimeFlags(parser):
  """Adds accelerator and machine type flags to the parser for switch."""
  accelerator_choices = [
      'NVIDIA_TESLA_A100', 'NVIDIA_TESLA_K80', 'NVIDIA_TESLA_P100',
      'NVIDIA_TESLA_V100', 'NVIDIA_TESLA_P4', 'NVIDIA_TESLA_T4',
      'NVIDIA_TESLA_T4_VWS', 'NVIDIA_TESLA_P100_VWS', 'NVIDIA_TESLA_P4_VWS',
      'TPU_V2', 'TPU_V3'
  ]
  AddRuntimeResource(parser)
  parser.add_argument('--machine-type', help=('machine type'))
  accelerator_config_group = parser.add_group()
  accelerator_config_group.add_argument(
      '--accelerator-type',
      help='Type of this accelerator.',
      choices=accelerator_choices,
      default=None)
  accelerator_config_group.add_argument(
      '--accelerator-core-count',
      help='Count of cores of this accelerator.',
      type=int)


def AddResetRuntimeFlags(parser):
  AddRuntimeResource(parser)


def AddCreateRuntimeFlags(parser):
  """Construct groups and arguments specific to the runtime creation."""

  AddRuntimeResource(parser)
  runtime_type_group = parser.add_group(mutex=True, required=True)
  runtime_type_group.add_argument(
      '--runtime-type', help=('runtime type'))
  machine_type_group = runtime_type_group.add_group()
  machine_type_group.add_argument(
      '--machine-type', help=('machine type'), required=True)
  local_disk_group = machine_type_group.add_group()
  local_disk_group.add_argument(
      '--interface', help=('runtime interface'))
  local_disk_group.add_argument('--source', help=('runtime source'))
  local_disk_group.add_argument('--mode', help=('runtime mode'))
  local_disk_group.add_argument('--type', help=('runtime type'))

  access_config_group = parser.add_group(required=True)
  access_config_group.add_argument(
      '--runtime-access-type', help=('access type'))
  access_config_group.add_argument('--runtime-owner', help=('runtime owner'))

  software_config_group = parser.add_group()
  software_config_group.add_argument(
      '--idle-shutdown-timeout', help=('idle shutdown timeout'))
  software_config_group.add_argument(
      '--install-gpu-driver', help=('install gpu driver'))
  software_config_group.add_argument(
      '--custom-gpu-driver-path', help=('custom gpu driver path'))
  software_config_group.add_argument(
      '--post-startup-script', help=('post startup script'))
  software_config_group.add_argument(
      '--post-startup-script-behavior', help=('post startup script behavior'))


def AddListRuntimeFlags(parser):
  GetLocationResourceArg(
      ('Location of this runtime. '
       'For example, us-central1-a')).AddToParser(parser)




# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to create an Attached cluster."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.container.gkemulticloud import attached as api_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.attached import flags as attached_flags
from googlecloudsdk.command_lib.container.attached import resource_args
from googlecloudsdk.command_lib.container.gkemulticloud import command_util
from googlecloudsdk.command_lib.container.gkemulticloud import constants
from googlecloudsdk.command_lib.container.gkemulticloud import endpoint_util
from googlecloudsdk.command_lib.container.gkemulticloud import flags

# Command needs to be in one line for the docgen tool to format properly.
_EXAMPLES = """
To create a cluster named ``my-cluster'' managed in location ``us-west1'',
run:

$ {command} my-cluster --location=us-west1 --platform-version=PLATFORM_VERSION
"""


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Create(base.CreateCommand):
  """Create an Attached cluster."""

  detailed_help = {'EXAMPLES': _EXAMPLES}

  @staticmethod
  def Args(parser):
    """Registers flags for this command."""
    resource_args.AddAttachedClusterResourceArg(parser, 'to create')

    attached_flags.AddPlatformVersion(parser)
    attached_flags.AddOidcConfig(parser)
    attached_flags.AddDistribution(parser, required=True)
    attached_flags.AddAdminUsers(parser)

    flags.AddAnnotations(parser)
    flags.AddValidateOnly(parser, 'cluster to create')
    flags.AddFleetProject(parser)
    flags.AddDescription(parser)

    base.ASYNC_FLAG.AddToParser(parser)

    parser.display_info.AddFormat(constants.ATTACHED_CLUSTERS_FORMAT)

  def Run(self, args):
    """Runs the create command."""
    location = resource_args.ParseAttachedClusterResourceArg(args).locationsId
    with endpoint_util.GkemulticloudEndpointOverride(location):
      cluster_ref = resource_args.ParseAttachedClusterResourceArg(args)
      cluster_client = api_util.ClustersClient()
      message = command_util.ClusterMessage(
          cluster_ref.attachedClustersId,
          action='Creating',
          kind=constants.ATTACHED)
      return command_util.Create(
          resource_ref=cluster_ref,
          resource_client=cluster_client,
          args=args,
          message=message,
          kind=constants.ATTACHED_CLUSTER_KIND)
